import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:syncrow_web/common/edit_chip.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/domain/models/space_details_model.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/presentation/widgets/button_content_widget.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/presentation/widgets/assign_tags_dialog.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/constants/assets.dart';
import 'package:syncrow_web/utils/enum/device_types.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';

class SpaceDetailsDevicesBox extends StatelessWidget {
  const SpaceDetailsDevicesBox({
    required this.space,
    super.key,
  });

  final SpaceDetailsModel space;

  @override
  Widget build(BuildContext context) {
    final productAllocations = space.productAllocations;
    final subspaces = space.subspaces;
    final isAnySubspaceHasProductAllocations =
        subspaces.any((subspace) => subspace.productAllocations.isNotEmpty);
    if (productAllocations.isNotEmpty || isAnySubspaceHasProductAllocations) {
      return Container(
        width: double.infinity,
        padding: const EdgeInsets.all(8),
        decoration: BoxDecoration(
          color: ColorsManager.textFieldGreyColor,
          borderRadius: BorderRadius.circular(15),
          border: Border.all(
            color: ColorsManager.textFieldGreyColor,
            width: 3.0,
          ),
        ),
        child: Wrap(
          spacing: 8.0,
          runSpacing: 8.0,
          children: [
            ...productAllocations.map(
              (entry) => Chip(
                avatar: SizedBox(
                  width: 24,
                  height: 24,
                  child: SvgPicture.asset(
                    _getDeviceIcon(entry.product.productType),
                    fit: BoxFit.contain,
                  ),
                ),
                label: Text(
                  entry.product.productType,
                  style: context.textTheme.bodySmall
                      ?.copyWith(color: ColorsManager.spaceColor),
                ),
                backgroundColor: ColorsManager.whiteColors,
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(16),
                  side: const BorderSide(
                    color: ColorsManager.spaceColor,
                  ),
                ),
              ),
            ),
            EditChip(
              onTap: () => _showAssignTagsDialog(context),
            ),
          ],
        ),
      );
    } else {
      return TextButton(
        onPressed: () => _showAssignTagsDialog(context),
        style: TextButton.styleFrom(
          padding: EdgeInsets.zero,
        ),
        child: const SizedBox(
          width: double.infinity,
          child: ButtonContentWidget(
            svgAssets: Assets.addIcon,
            label: 'Add Devices',
          ),
        ),
      );
    }
  }

  void _showAssignTagsDialog(BuildContext context) {
    showDialog<void>(
      context: context,
      builder: (context) => AssignTagsDialog(space: space),
    );
  }

  String _getDeviceIcon(String productType) =>
      switch (devicesTypesMap[productType]) {
        DeviceType.LightBulb => Assets.lightBulb,
        DeviceType.CeilingSensor => Assets.sensors,
        DeviceType.AC => Assets.ac,
        DeviceType.DoorLock => Assets.doorLock,
        DeviceType.Curtain => Assets.curtain,
        DeviceType.ThreeGang => Assets.gangSwitch,
        DeviceType.Gateway => Assets.gateway,
        DeviceType.OneGang => Assets.oneGang,
        DeviceType.TwoGang => Assets.twoGang,
        DeviceType.WH => Assets.waterHeater,
        DeviceType.DoorSensor => Assets.openCloseDoor,
        DeviceType.GarageDoor => Assets.openedDoor,
        DeviceType.WaterLeak => Assets.waterLeakNormal,
        DeviceType.Curtain2 => Assets.curtainIcon,
        DeviceType.Blind => Assets.curtainIcon,
        DeviceType.WallSensor => Assets.sensors,
        DeviceType.DS => Assets.openCloseDoor,
        DeviceType.OneTouch => Assets.gangSwitch,
        DeviceType.TowTouch => Assets.gangSwitch,
        DeviceType.ThreeTouch => Assets.gangSwitch,
        DeviceType.NCPS => Assets.sensors,
        DeviceType.PC => Assets.powerClamp,
        DeviceType.Other => Assets.blackLogo,
        null => Assets.blackLogo,
      };
}
