import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/space_details/domain/models/space_details_model.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/update_space/presentation/bloc/space_details_model_bloc/space_details_model_bloc.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';

class SubspaceNameDisplayWidget extends StatefulWidget {
  const SubspaceNameDisplayWidget({super.key, required this.subSpace});

  final Subspace subSpace;

  @override
  State<SubspaceNameDisplayWidget> createState() =>
      _SubspaceNameDisplayWidgetState();
}

class _SubspaceNameDisplayWidgetState extends State<SubspaceNameDisplayWidget> {
  late final TextEditingController _controller;
  late final FocusNode _focusNode;
  bool _isEditing = false;
  bool _hasDuplicateName = false;

  @override
  void initState() {
    _controller = TextEditingController(text: widget.subSpace.name);
    _focusNode = FocusNode();
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    _focusNode.dispose();
    super.dispose();
  }

  bool _checkForDuplicateName(String name) {
    final bloc = context.read<SpaceDetailsModelBloc>();
    return bloc.state.subspaces
        .where((s) => s.uuid != widget.subSpace.uuid)
        .any((s) => s.name.toLowerCase() == name.toLowerCase());
  }

  void _handleNameChange(String value) {
    setState(() {
      _hasDuplicateName = _checkForDuplicateName(value);
    });
  }

  void _tryToFinishEditing() {
    if (!_hasDuplicateName) {
      _onFinishEditing();
    }
  }

  void _tryToSubmit(String value) {
    if (_hasDuplicateName) return;

    final bloc = context.read<SpaceDetailsModelBloc>();
    bloc.add(
      UpdateSpaceDetailsSubspaces(
        bloc.state.subspaces
            .map(
              (e) => e.uuid == widget.subSpace.uuid ? e.copyWith(name: value) : e,
            )
            .toList(),
      ),
    );
    _onFinishEditing();
  }

  @override
  Widget build(BuildContext context) {
    final textStyle = context.textTheme.bodySmall?.copyWith(
      color: ColorsManager.spaceColor,
    );
    return InkWell(
      onTap: () {
        setState(() => _isEditing = true);
        _focusNode.requestFocus();
      },
      child: Chip(
        backgroundColor: ColorsManager.whiteColors,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(20),
          side: const BorderSide(color: ColorsManager.transparentColor),
        ),
        onDeleted: () {
          final bloc = context.read<SpaceDetailsModelBloc>();
          bloc.add(
            UpdateSpaceDetailsSubspaces(
              bloc.state.subspaces
                  .where((s) => s.uuid != widget.subSpace.uuid)
                  .toList(),
            ),
          );
        },
        deleteIcon: Container(
          padding: const EdgeInsetsDirectional.all(1),
          decoration: BoxDecoration(
            shape: BoxShape.circle,
            border: Border.all(
              color: ColorsManager.lightGrayColor,
              width: 1.5,
            ),
          ),
          child: const FittedBox(
            child: Icon(
              Icons.close,
              color: ColorsManager.lightGrayColor,
            ),
          ),
        ),
        label: Visibility(
          visible: _isEditing,
          replacement: Text(
            widget.subSpace.name,
            style: textStyle,
          ),
          child: Column(
            mainAxisSize: MainAxisSize.min,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              SizedBox(
                width: context.screenWidth * 0.065,
                height: context.screenHeight * 0.025,
                child: TextField(
                  focusNode: _focusNode,
                  controller: _controller,
                  style: textStyle?.copyWith(
                    color: _hasDuplicateName ? Colors.red : null,
                  ),
                  decoration: const InputDecoration.collapsed(
                    hintText: '',
                  ),
                  onChanged: _handleNameChange,
                  onTapOutside: (_) => _tryToFinishEditing(),
                  onSubmitted: _tryToSubmit,
                ),
              ),
              if (_hasDuplicateName)
                AnimatedSwitcher(
                  duration: const Duration(milliseconds: 250),
                  child: Visibility(
                    key: ValueKey(_hasDuplicateName),
                    visible: _hasDuplicateName,
                    child: Text(
                      'Name already exists',
                      style: textStyle?.copyWith(
                        color: Colors.red,
                        fontSize: 8,
                      ),
                    ),
                  ),
                ),
            ],
          ),
        ),
      ),
    );
  }

  void _onFinishEditing() {
    setState(() {
      _isEditing = false;
      _hasDuplicateName = false;
    });
    _focusNode.unfocus();
  }
}
