import 'package:flutter/material.dart';
import 'package:syncrow_web/pages/space_management_v2/modules/tags/domain/models/tag.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';

class ProductTagField extends StatefulWidget {
  final List<Tag> items;
  final ValueChanged<Tag> onSelected;
  final Tag? initialValue;
  final String productName;

  const ProductTagField({
    super.key,
    required this.items,
    required this.onSelected,
    this.initialValue,
    required this.productName,
  });

  @override
  State<ProductTagField> createState() => _ProductTagFieldState();
}

class _ProductTagFieldState extends State<ProductTagField> {
  bool _isOpen = false;
  OverlayEntry? _overlayEntry;
  final TextEditingController _controller = TextEditingController();
  final FocusNode _focusNode = FocusNode();
  List<Tag> _filteredItems = [];

  @override
  void initState() {
    super.initState();
    _controller.text = widget.initialValue?.name ?? '';

    _filterItems();

    _focusNode.addListener(() {
      if (!_focusNode.hasFocus) {
        final selectedTag = _filteredItems.firstWhere(
          (tag) => tag.name == _controller.text,
          orElse: () => Tag(
            name: _controller.text,
            uuid: '',
            createdAt: '',
            updatedAt: '',
          ),
        );
        widget.onSelected(selectedTag);
        _closeDropdown();
      }
    });
  }

  @override
  void dispose() {
    _controller.dispose();
    _focusNode.dispose();
    _overlayEntry = null;
    _isOpen = false;
    super.dispose();
  }

  void _filterItems() => setState(() => _filteredItems = widget.items);

  void _toggleDropdown() {
    if (_isOpen) {
      _closeDropdown();
    } else {
      _openDropdown();
    }
  }

  void _openDropdown() {
    _overlayEntry = _createOverlayEntry();
    Overlay.of(context).insert(_overlayEntry!);
    _isOpen = true;
  }

  void _closeDropdown() {
    if (_isOpen && _overlayEntry != null) {
      _overlayEntry!.remove();
      _overlayEntry = null;
      _isOpen = false;
    }
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () => FocusScope.of(context).unfocus(),
      behavior: HitTestBehavior.opaque,
      child: Container(
        padding: const EdgeInsets.symmetric(horizontal: 12.0, vertical: 8.0),
        decoration: BoxDecoration(
          border: Border.all(color: ColorsManager.transparentColor),
          borderRadius: BorderRadius.circular(8.0),
        ),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Expanded(
              child: TextFormField(
                controller: _controller,
                focusNode: _focusNode,
                onFieldSubmitted: (value) {
                  final selectedTag = _filteredItems.firstWhere(
                      (tag) => tag.name == value,
                      orElse: () =>
                          Tag(name: value, uuid: '', createdAt: '', updatedAt: ''));
                  widget.onSelected(selectedTag);
                  _closeDropdown();
                },
                onTapOutside: (event) {
                  widget.onSelected(_filteredItems.firstWhere(
                      (tag) => tag.name == _controller.text,
                      orElse: () => Tag(
                          name: _controller.text,
                          uuid: '',
                          createdAt: '',
                          updatedAt: '')));
                  _closeDropdown();
                },
                style: context.textTheme.bodyMedium,
                decoration: const InputDecoration(
                  hintText: 'Enter or Select a tag',
                  border: InputBorder.none,
                ),
              ),
            ),
            GestureDetector(
              onTap: _toggleDropdown,
              child: const Icon(Icons.arrow_drop_down),
            ),
          ],
        ),
      ),
    );
  }

  OverlayEntry _createOverlayEntry() {
    final renderBox = context.findRenderObject()! as RenderBox;
    final size = renderBox.size;
    final offset = renderBox.localToGlobal(Offset.zero);

    return OverlayEntry(
      builder: (context) {
        return GestureDetector(
          onTap: _closeDropdown,
          behavior: HitTestBehavior.translucent,
          child: Stack(
            children: [
              Positioned(
                left: offset.dx,
                top: offset.dy + size.height,
                width: size.width,
                child: Material(
                  elevation: 4.0,
                  child: Container(
                    color: ColorsManager.whiteColors,
                    constraints: const BoxConstraints(maxHeight: 200.0),
                    child: StatefulBuilder(
                      builder: (context, setStateDropdown) {
                        return ListView.builder(
                          shrinkWrap: true,
                          itemCount: _filteredItems.length,
                          itemBuilder: (context, index) {
                            final tag = _filteredItems[index];

                            return Container(
                              decoration: const BoxDecoration(
                                border: Border(
                                  bottom: BorderSide(
                                    color: ColorsManager.lightGrayBorderColor,
                                    width: 1.0,
                                  ),
                                ),
                              ),
                              child: ListTile(
                                title: Text(
                                  tag.name,
                                  style: context.textTheme.bodyMedium?.copyWith(
                                    color: ColorsManager.textPrimaryColor,
                                  ),
                                ),
                                onTap: () {
                                  _controller.text = tag.name;
                                  widget.onSelected(tag);
                                  setState(() {
                                    _filteredItems.remove(tag);
                                  });
                                  _closeDropdown();
                                },
                              ),
                            );
                          },
                        );
                      },
                    ),
                  ),
                ),
              ),
            ],
          ),
        );
      },
    );
  }
}
